#!/bin/sh
#
# GNU/Fiwix installation script.
# Copyright 2018, Jordi Sanfeliu. All rights reserved.
# Distributed under the terms of the Fiwix License.
#

VERSION=$(uname -r)
MAXPARTSIZE=1070
umask 0022

RED="echo -en \\033[1;31m"
GREEN="echo -en \\033[1;32m"
YELLOW="echo -en \\033[1;33m"
CYAN="echo -en \\033[1;36m"
BLUE="echo -en \\033[1;34m"
BGBLUE="echo -en \\033[1;44m"
DEFAULT="echo -en \\033[m"	# \\033[0;39m
REVERSE="echo -en \\033[7m"
BRIGHT="echo -en \\033[1;39m"

LINES=24
keymap=

scroll()
{
	local l
	for l in $(seq 1 1 $LINES) ; do
		echo
	done
}

header() {
	echo "	GNU/Fiwix Operating System Installation"
	echo "	--------------------------------------------------------------"
	echo
	echo
}

press_enter()
{
	local in
	$REVERSE "Press <ENTER> to $1"
	$DEFAULT
	$BRIGHT
	$BGBLUE
	read in
}

welcome()
{
	echo "				  W E L C O M E"
	echo
	echo "		to the installation of GNU/Fiwix Operating System"
	echo "			      for i386 architecture"
	echo
	echo "		Copyright (c) 2018 by Jordi Sanfeliu (aka Mikaku)"
	echo "				<jordi@fiwix.org>"
	echo
	echo
	echo "				kernel v$VERSION"
	echo
	echo
	echo
	echo
	echo
	echo
	echo
	press_enter "continue"
}

info1()
{
	echo "	Fiwix is an operating system kernel, written by Jordi Sanfeliu"
	echo "	from scratch, based on the UNIX architecture and fully focused"
	echo "	on being POSIX compatible. It is designed and developed mainly"
	echo "	as a hobby OS but also for educational purposes, therefore the"
	echo "	kernel code is kept as simple as possible."
	echo
	echo "	This first version of Fiwix  is designed to run on most 32-bit"
	echo "	ISA PCs in existence. So it must run even on an 80386 with 2MB"
	echo "	of RAM (although the minimum recommended is 64MB)."
	echo
	echo "	Please keep in mind  that this  kernel is not yet  suited  for"
	echo "	production  use and  may  well  have serious  bugs  and broken"
	echo "	features which have not yet been identified or resolved."
	echo
	echo
	echo
	echo "			*****************************"
	echo "			*** USE AT YOUR OWN RISK! ***"
	echo "			*****************************"
	echo
	press_enter "continue"
}

info2()
{
	echo "	The installation  program  will  start  shortly.  You  will be"
	echo "	prompted for the information needed to install  the  GNU/Fiwix"
	echo "	Operating System  in your hard disk drive.  Default values are"
	echo "	provided where useful."
	echo
	echo "	You can interrupt the  installation  procedure at  any time by"
	echo "	pressing  <CTRL-C>,  then  type  'install.sh'  and  start  the"
	echo "	installation again."
	echo
	echo
	echo "	Please be  patient and  read the  instructions  on  the screen"
	echo "	carefully."
	echo
	echo "	Thanks."
	echo
	echo
	echo
	echo
	echo
	echo
	press_enter "continue"
}

error()
{
	echo
	echo
	echo "An error occurred during the installation."
	echo
	press_enter "exit"
	return 1
}

load_keymap()
{
	case $1 in
		1) keymap=cf.bmap ;;
		2) keymap=dk.bmap ;;
		3) keymap=nl.bmap ;;
		4) keymap=fi.bmap ;;
		5) keymap=fr.bmap ;;
		6) keymap=de.bmap ;;
		7) keymap=gr.bmap ;;
		8) keymap=hu.bmap ;;
		9) keymap=it.bmap ;;
		10) keymap=no.bmap ;;
		11) keymap=pl.bmap ;;
		12) keymap=pt.bmap ;;
		13) keymap=es.bmap ;;
		14) keymap=uk.bmap ;;
		15) keymap=us.bmap ;;
		*) return 1 ;;
	esac

	loadkmap </etc/i18n/$keymap
	return 0
}

verify_media()
{
	[ "$BOOTDEV" = "$1" ] && return 0
	$DEFAULT
	if ! mount -t iso9660 "$1" /mnt/cdrom ; then
		sleep 2
		return 1
	fi

	$BRIGHT ; $BGBLUE
	if ! test -d /mnt/cdrom/install/pkgs ; then
		echo
		echo "This media is not the GNU/Fiwix Installation CD-ROM"
		sleep 2
		umount /mnt/cdrom
		return 1
	fi

	sleep 2
	return 0
}

makedev() {
	mknod /mnt/disk/dev/$1 $2 $3 $4
#	chmod $5 /mnt/disk/dev/$1
}


# Main
# -----------------------------------------------------------------------------

$BRIGHT ; $BGBLUE
scroll ; welcome
scroll ; header ; info1
scroll ; header ; info2

BOOTDEV=$(df / | grep dev | awk '{print $1}')
HDLIST=$(grep " hd.$" /proc/partitions | awk '{print "/dev/"$4}')

if [ -z "$HDLIST" ] ; then
	scroll
	echo "Sorry,  no hard disk drives have been detected in your system."
	echo
	echo
	echo "Please, verify you have an IDE/ATA compatible  hard  disk  and"
	echo "restart the installation procedure."
	echo
	echo "Thanks."
	echo
	echo
	echo
	press_enter "exit"
	$DEFAULT
	exit 1
fi


# keyboard layout selection
while : ; do
	scroll
	echo
	echo "Keyboard Selection"
	echo
	echo "Please specify the keyboard layout from the list below."
	echo
	echo "1. Canadian-French		 9. Italian"
	echo "2. Danish			10. Norwegian"
	echo "3. Dutch			11. Polish"
	echo "4. Finish			12. Portuguese"
	echo "5. French			13. Spanish-Catalan"
	echo "6. German			14. UK-English"
	echo "7. Greece			15. US-English"
	echo "8. Hungarian"
	echo

	echo -n "Enter the number of the appropiate layout: "
	read kbd

	load_keymap $kbd && break
done


# source media selection (unless it was booted from the CD-ROM)
if ! grep -q "/dev/root / iso9660" /proc/mounts ; then
while : ; do
	scroll
	echo "SOURCE MEDIA SELECTION"
	echo
	echo "The installation program needs to know from which source media"
	echo "(likely a CD-ROM device) will you be installing GNU/Fiwix:"
	echo

	echo -n "Type the appropiate device (e.g. /dev/hdc): "
	read src
	echo

	[ -z "$src" ] && continue
	verify_media $src && break
done
fi


# disk media selection
$BRIGHT ; $BGBLUE
while : ; do
	scroll
	echo "*****  IMPORTANT!  READ THE INFORMATION BELOW CAREFULLY  *****"
	echo
	echo "You need to specify in which hard disk you want to install the"
	echo "GNU/Fiwix Operating System.  A minimal of 500MB of capacity is"
	echo "required."
	echo
	echo "The following is the list of hard disk drives detected in your"
	echo "system:"
	echo

	for disk in $HDLIST ; do
		hd=$(echo $disk | sed 's#/dev/##')
		kb=$(grep "$hd$" /proc/partitions | awk '{print $3}')
		[ -n "$kb" ] && disksize=$(($kb / 1024))
		echo -en "\t$disk (${disksize}MB)"
		if [ "$disksize" -lt 500 ] ; then
			echo "  INSUFFICIENT CAPACITY!"
		else
			[ -z "$default" ] && default="$disk"
			echo
		fi
	done
	echo

	echo -n "Type the appropiate device [$default]: "
	read target

	[ -z "$target" ] && target=$default
	for disk in $HDLIST ; do
		if [ "$disk" = "$target" ] ; then
			hd=$(echo $disk | sed 's#/dev/##')
			kb=$(grep "$hd$" /proc/partitions | awk '{print $3}')
			disksize=$(($kb / 1024))
			break;
		fi
	done

	# default values for number of partitions and sizes
	nparts=2
	bootsize=8
	rootsize=max
	homesize=max
	usrsize=max

	disksize=$(($disksize - ($bootsize + 1)))
	if [ "$disksize" -gt $MAXPARTSIZE ] ; then
		nparts=3
		rootsize=$MAXPARTSIZE
		disksize=$(($disksize - $rootsize))
	fi
	if [ "$disksize" -gt $MAXPARTSIZE ] ; then
		nparts=4
		homesize=$MAXPARTSIZE
		disksize=$(($disksize - $homesize))
	fi
	if [ "$disksize" -gt $MAXPARTSIZE ] ; then
		nparts=4
		usrsize=$MAXPARTSIZE
	fi

	echo
	echo
	echo "WARNING:  If you continue all files currently on the hard disk"
	echo "'$target' will be erased!"
	echo

	while : ; do
		echo -n "Do you really want to continue ('yes' or 'no')? "
		read in
		[ "$in" = "no" ] && break
		[ "$in" = "yes" ] && break 2
	done
done


	# RAMdisk is used to store temporary files in /tmp
	echo
	echo
	echo "Initializing RAMdisk device"
	$DEFAULT
	mkfs.minix -n 30 /dev/ram || error || exit 1
	mount -t minix /dev/ram /mnt/ram || error || exit 1
	mkdir /mnt/ram/tmp

if grep -q "root=/dev/fd0" /proc/cmdline ; then
	SOURCE=/mnt/cdrom
else
	# if installing from the CDROM source is /
	SOURCE=
fi

echo
$BRIGHT ; $BGBLUE ; echo "Initializing disk partition block"
$DEFAULT ; dd if=/dev/zero of=$target bs=1c count=512 >/dev/null || error || exit 1
echo
$DEFAULT ; echo w | fdisk $target >/dev/null || error || exit 1
echo

$BRIGHT ; $BGBLUE ; echo "Creating partition 1 for /boot"
$DEFAULT ; (echo n ; echo p ; echo 1 ; echo 1 ; echo +${bootsize}M ; echo w) | fdisk $target >/dev/null || error || exit 1
$BRIGHT ; $BGBLUE ; echo "Creating partition 2 for /"
if [ "$rootsize" = "max" ] ; then
	$DEFAULT ; (echo n ; echo p ; echo 2 ; echo ; echo ; echo w) | fdisk $target >/dev/null || error || exit 1
else
	$DEFAULT ; (echo n ; echo p ; echo 2 ; echo ; echo +${rootsize}M ; echo w) | fdisk $target >/dev/null || error || exit 1
fi
if [ "$nparts" -gt 2 ] ; then
	$BRIGHT ; $BGBLUE ; echo "Creating partition 3 for /home"
	if [ "$homesize" = "max" ] ; then
		$DEFAULT ; (echo n ; echo p ; echo 3 ; echo ; echo ; echo w) | fdisk $target >/dev/null || error || exit 1
	else
		$DEFAULT ; (echo n ; echo p ; echo 3 ; echo ; echo +${homesize}M ; echo w) | fdisk $target >/dev/null || error || exit 1
	fi
	if [ "$nparts" -gt 3 ] ; then
		$BRIGHT ; $BGBLUE ; echo "Creating partition 4 for /usr"
		if [ "$usrsize" = "max" ] ; then
			$DEFAULT ; (echo n ; echo p ; echo 3 ; echo ; echo ; echo w) | fdisk $target >/dev/null || error || exit 1
		else
			$DEFAULT ; (echo n ; echo p ; echo 4 ; echo ; echo +${usrsize}M ; echo w) | fdisk $target >/dev/null || error || exit 1
		fi
	fi
fi

$BRIGHT ; $BGBLUE ; echo "Making filesystems, this may take a while"
$DEFAULT ; (echo t ; echo 1 ; echo 81 ; echo w) | fdisk $target >/dev/null || error || exit 1
$DEFAULT ; mkfs.minix -n 30 ${target}1 >/dev/null || error || exit 1
$BRIGHT ; $BGBLUE ; echo "/boot ... done"
$DEFAULT ; (echo t ; echo 2 ; echo 81 ; echo w) | fdisk $target >/dev/null || error || exit 1
$DEFAULT ; mkfs.minix -n 30 -v ${target}2 >/dev/null || error || exit 1
$BRIGHT ; $BGBLUE ; echo "/ ... done"
if [ "$nparts" -gt 2 ] ; then
	$DEFAULT ; (echo t ; echo 3 ; echo 81 ; echo w) | fdisk $target >/dev/null || error || exit 1
	$DEFAULT ; mkfs.minix -n 30 -v ${target}3 >/dev/null || error || exit 1
	$BRIGHT ; $BGBLUE ; echo "/home ... done"
	if [ "$nparts" -gt 3 ] ; then
		$DEFAULT ; (echo t ; echo 4 ; echo 81 ; echo w) | fdisk $target >/dev/null || error || exit 1
		$DEFAULT ; mkfs.minix -n 30 -v ${target}4 >/dev/null || error || exit 1
		$BRIGHT ; $BGBLUE ; echo "/usr ... done"
	fi
fi

echo
echo
$BRIGHT ; $BGBLUE ; echo "The following is the current partition layout in $target"
fdisk -l $target
echo
press_enter "continue"


# system installation
scroll
echo "SOFTWARE INSTALLATION"
echo
echo "All software packages in CD-ROM are going to be transferred on"
echo "to your hard disk."
echo
press_enter "continue"


scroll
echo "Preparing root filesystem ..."
$DEFAULT ; mount -t minix ${target}2 /mnt/disk || error || exit 1

cd /mnt/disk
$BRIGHT ; $BGBLUE ; echo "creating memory devices ..."
mkdir /mnt/disk/dev
makedev mem c 1 1
makedev kmem c 1 2
makedev null c 1 3
makedev zero c 1 5
makedev ram0 b 1 0
ln -s dev/ram0 dev/ram
echo "creating floppy devices ..."
makedev fd0 b 2 0
makedev fd0h360 b 2 4
makedev fd0h1200 b 2 8
makedev fd0h720 b 2 12
makedev fd0h1440 b 2 16
makedev fd1 b 2 1
makedev fd1h360 b 2 5
makedev fd1h1200 b 2 9
makedev fd1h720 b 2 13
makedev fd1h1440 b 2 17
echo "creating hard disk devices ..."
makedev hda b 3 0
makedev hda1 b 3 1
makedev hda2 b 3 2
makedev hda3 b 3 3
makedev hda4 b 3 4
makedev hdb b 3 64
makedev hdb1 b 3 65
makedev hdb2 b 3 66
makedev hdb3 b 3 67
makedev hdb4 b 3 68
makedev hdc b 22 0
makedev hdc1 b 22 1
makedev hdc2 b 22 2
makedev hdc3 b 22 3
makedev hdc4 b 22 4
makedev hdd b 22 64
makedev hdd1 b 22 65
makedev hdd2 b 22 66
makedev hdd3 b 22 67
makedev hdd4 b 22 68
echo "creating tty devices ..."
makedev console c 5 1
makedev tty c 5 0
makedev tty0 c 4 0
makedev tty1 c 4 1
makedev tty2 c 4 2
makedev tty3 c 4 3
makedev tty4 c 4 4
makedev tty5 c 4 5
makedev tty6 c 4 6
makedev tty7 c 4 7
makedev tty8 c 4 8
makedev tty9 c 4 9
makedev tty10 c 4 10
makedev tty11 c 4 11
makedev tty12 c 4 12
echo "creating lp0 device ..."
makedev lp0 c 6 0
echo "creating fifo devices ..."
mkfifo dev/fifo

echo "creating system directories ..."
mkdir bin
mkdir boot
mkdir home
mkdir mnt
mkdir mnt/cdrom
mkdir mnt/disk
mkdir mnt/floppy
mkdir proc
mkdir root
mkdir sbin
mkdir tmp
mkdir -p usr/sbin
mkdir -p var/run
chmod 1777 tmp

if [ "$nparts" -gt 3 ] ; then
	$DEFAULT ; mount -t minix ${target}4 /mnt/disk/usr || error || exit 1
fi

$BRIGHT ; $BGBLUE ; echo "Installing software, please wait ..."
for file in $(ls ${SOURCE}/install/pkgs/bin/*.tar.gz) ; do
	$BRIGHT ; $BGBLUE ; echo -e "\t$file"
	$DEFAULT ; tar zxf $file
done
for file in $(ls ${SOURCE}/install/base/*.tar.gz) ; do
	$BRIGHT ; $BGBLUE ; echo -e "\t$file"
	$DEFAULT ; tar zxf $file
done
chown -R root:root etc sbin usr 2>/dev/null
cp ${SOURCE}/README root
chmod 700 root

cat <<EOF >root/.bash_profile
# .bash_profile

# Get the aliases and functions
if [ -f ~/.bashrc ] ; then
	. ~/.bashrc
fi

# User specific environment and startup programs
PATH=$PATH:$HOME/bin

export PATH

EOF

cat <<EOF >etc/fstab
${target}2	/	minix	defaults	1 1
${target}1	/boot	minix	ro,defaults	0 0
EOF

if [ "$nparts" -gt 2 ] ; then
	cat <<EOF >>etc/fstab
${target}3	/home	minix	rw,defaults	0 0
EOF
fi

if [ "$nparts" -gt 3 ] ; then
	cat <<EOF >>etc/fstab
${target}4	/usr	minix	rw,defaults	0 0
EOF
fi

cat <<EOF >>etc/fstab
none		/proc	proc	defaults	0 0
EOF

touch etc/mtab

cat <<EOF >>etc/motd

Welcome to GNU/Fiwix v$VERSION for i386 architecture.
Copyright 2018, Jordi Sanfeliu.
Distributed under the terms of the Fiwix License.
All rights reserved.

    - Read the /README file to know information about all the files installed.
    - Refer to the CDROM for the source code and the license of every package
      installed in this system.

GNU/Fiwix comes with ABSOLUTELY NO WARRANTY.  *** USE AT YOUR OWN RISK! ***

EOF

$BRIGHT ; $BGBLUE ; echo "Installing Busybox, please wait ..."
$DEFAULT
cp ${SOURCE}/install/base/busybox sbin
ln -s /sbin/busybox bin/ash
ln -s /sbin/busybox bin/free
ln -s /sbin/busybox bin/login
ln -s /sbin/busybox bin/mount
ln -s /sbin/busybox bin/ps
ln -s /sbin/busybox bin/umount
ln -s /sbin/busybox sbin/fdisk
ln -s /sbin/busybox sbin/fsck.minix
ln -s /sbin/busybox sbin/getty
ln -s /sbin/busybox sbin/halt
ln -s /sbin/busybox sbin/init
ln -s /sbin/busybox sbin/loadkmap
ln -s /sbin/busybox sbin/mkfs.minix
ln -s /sbin/busybox sbin/poweroff
ln -s /sbin/busybox sbin/reboot

echo "loadkmap </etc/i18n/$keymap" >> etc/rc.d/rc.local

cd /
sync ; sync
if [ "$nparts" -gt 3 ] ; then
	umount /mnt/disk/usr || error || exit 1
fi
umount /mnt/disk || error || exit 1
mount -t minix ${target}1 /mnt/boot || error || exit 1


# installing GRUB
echo
$BRIGHT ; $BGBLUE ; echo "Setting up system ..."
$DEFAULT
ln -sf ${SOURCE}/install/grub/sbin/* /usr/sbin/ 2>/dev/null
ln -sf ${SOURCE}/install/grub/usr/lib /usr/ 2>/dev/null

mkdir -p /mnt/boot/grub
cp ${SOURCE}/install/grub/boot/fiwix.xpm.gz /mnt/boot/grub

cat <<EOF >/mnt/boot/grub/device.map
# this device map was generated during the GNU/Fiwix installation
(hd0)     /dev/hda
EOF

cat <<EOF >/mnt/boot/grub/grub.conf
# grub.conf generated during the GNU/Fiwix installation
#
# GNU/Fiwix Operating System
# Copyright (C) 2018 Jordi Sanfeliu <jordi@fiwix.org>
#
# GRUB configuration file
#
default 0
timeout 10
splashimage (hd0,0)/grub/fiwix.xpm.gz
title GNU/Fiwix $VERSION
	root (hd0,0)
	kernel /fiwix root=${target}2
EOF
ln -s grub.conf /mnt/boot/grub/menu.lst

$BRIGHT ; $BGBLUE ; echo "Installing GRUB on $target"
sync ; sync
$DEFAULT
grub-install --root-directory=/mnt $target || error || exit 1

# cleaning up
rm -f /mnt/boot/grub/default
rm -f /usr/lib 2>/dev/null
rm -f /usr/sbin/grub* 2>/dev/null

cp /boot/fiwix /mnt/boot
gzip -dc /boot/System.map.gz > /mnt/boot/System.map-$VERSION
ln -s System.map-$VERSION /mnt/boot/System.map

if ! grep -q "/dev/root / iso9660" /proc/mounts ; then
	umount /mnt/cdrom
fi
umount /mnt/boot
umount /mnt/ram
sync ; sync

scroll
$BRIGHT ; $BGBLUE
echo "Installation of GNU/Fiwix v$VERSION is complete."
echo
echo "When you press ENTER, the system will be shut down and restarted."
echo
echo "*** MAKE SURE THE BOOT DRIVES (DISKETTE AND/OR CD-ROM) ARE EMPTY ***"
echo
press_enter "reboot"

$DEFAULT
reboot

